/* Take raw data from the Finnish wealth survey and smooth the relation between income and house value
 used in "Income Distribution and Housing Prices: An Assignment Model Approach" by
 Niku Määttänen and Marko Terviö (HECER Discussion Paper)
*/

* Input 
global INPATH "helsinki_04_varallisuudata.csv" // raw micro data (proprietory)
global OUTPATH "helsinki_2004.csv"  // smoothed distribution used in the calibration

* Settings
global rate = 0.05 // discount rate
global bottomCodePercent = 5 // in %, dropping lowest reported house values 
global K=1000 // number of points for the house value grid
global topCodePercent = 0 // not needed in Finnish data

* PREPROCESS DATA

import delimited $INPATH, case(preserve) clear 

gen income_raw = KTURAHA + ASUVELAT*$rate 	// permanent income 
gen value_raw =  ASARAV 		// house value
sort value_raw
local nKeep = round((1-$topCodePercent/100)*_N )
keep in 1/`nKeep'
egen rank = rank(value_raw), field

* KERNEL REGRESSION  

* Find the smallest bandwidth that makes wealth-house value relation strictly monotonic
local bw = 10
local violations =1
while `violations'>0 {
	preserve
	display as result "Bandwidth: " `bw' ""
	quietly lpoly income_raw rank if income_raw>0, at(rank) degree(1) bwidth(`bw') gen(fit) nograph
	quietly sum rank
	quietly gen quantile = 1 - (rank-0.5)/r(max)
	quietly sort quantile
	quietly	collapse (count) m = income_raw (mean) fit value_raw income_raw, by(quantile)
	count if fit[_n] < fit[_n-1] & (fit[_n-1]!=.) 
	local violations = `r(N)' 	// number of monotonicity violations
	restore
	global useBW =`bw'
	local bw = `bw'+1
}


di "Smallest bandwidth that results in monotonic relation: $useBW" _n "(discount rate $rate)" 

lpoly income_raw rank if income_raw>0, kernel(epa) at(rank) degree(1) bwidth($useBW) gen(fit) nograph
quietly sum rank
quietly gen quantile=1-(rank-0.5)/r(max)
sort quantile
collapse (count) m=income_raw (mean) fit value_raw income_raw, by(quantile)

* NB: the initial grid between quantiles is uneven, due to repeated house values. 


* INTERPOLATION to achieve an evenly spaced grid with $K points

sort quantile
count if quantile<$bottomCodePercent/100
local bottomN = `r(N)'-1
drop in 1/`bottomN'

sum quantile
local minQuantile = $bottomCodePercent/100
local maxQuantile = min(`r(max)',1-$topCodePercent/100)
local oldN = _N + 1
local newN = _N + $K  
set obs `newN'
forvalues i = `oldN'/`newN' {
	local newq = `minQuantile'+(`maxQuantile'-`minQuantile')*(`i'-`oldN')/($K-1)
	di `newq'
	replace quantile = `newq' in `i' 

}
di "`maxQuantile' `minQuantile'"

ipolate fit quantile, gen(income) 
ipolate value_raw quantile, gen(value) 
drop if value_raw!=.
drop *_raw fit m


* SAVE RESULTS

sort quantile 
order income quantile value 
export delimited income quantile value using $OUTPATH, replace 
